@@ -2,7 +2,7 @@ package Tk::ColorSelect;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '3.026'; # $Id: //depot/Tk8/Tk/ColorEditor.pm#26$
+$VERSION = '3.031'; # $Id: //depot/Tk8/Tk/ColorEditor.pm#31 $
 
 use Tk qw(Ev);
 
@@ -15,7 +15,12 @@ sub Populate
 {
     my ($middle,$args) = @_;
     my($i, @a);
-    foreach $i ('/usr/local/lib/X11/rgb.txt', '/usr/lib/X11/rgb.txt',
+    require Tk::Config;
+    my(@xlibpath) = map { s/^-L//; "$_/X11/rgb.txt" }
+                    split /\s+/, $Tk::Config::xlib;
+    foreach $i (@xlibpath,
+		'/usr/local/lib/X11/rgb.txt', '/usr/lib/X11/rgb.txt',
+		'/usr/X11R6/lib/X11/rgb.txt',
                 '/usr/local/X11R5/lib/X11/rgb.txt', '/X11/R5/lib/X11/rgb.txt',
                 '/X11/R4/lib/rgb/rgb.txt', '/usr/openwin/lib/X11/rgb.txt') {
         local *FOO;
@@ -208,7 +213,7 @@ sub color_space {
         $objref->{'Labels'}[$i] = $Labels{$space}->[$i];
        }
       $objref->{'color_space'} = $space;
-      $objref->DoWhenIdle(['set_scales',$objref]) unless ($objref->{'pending'}++);
+      $objref->afterIdle(['set_scales',$objref]) unless ($objref->{'pending'}++);
      }
  return $objref->{'color_space'};
 } # color_space
@@ -283,7 +288,7 @@ sub color
    my $hex = sprintf('#%04x%04x%04x', $red, $green, $blue);
    $objref->{'color'} = $hex;
    $objref->{'Entry'} = $name;
-   $objref->DoWhenIdle(['set_scales',$objref]) unless ($objref->{'pending'}++);
+   $objref->afterIdle(['set_scales',$objref]) unless ($objref->{'pending'}++);
    $objref->{'swatch'}->itemconfigure($objref->{'swatch_item'},
             -fill => $objref->{'color'});
   }
@@ -437,7 +442,7 @@ sub Show
 package Tk::ColorEditor;
 
 use vars qw($VERSION $SET_PALETTE);
-$VERSION = '3.026'; # $Id: //depot/Tk8/Tk/ColorEditor.pm#26$
+$VERSION = '3.031'; # $Id: //depot/Tk8/Tk/ColorEditor.pm#31 $
 
 use Tk qw(lsearch Ev);
 use Tk::Toplevel;
@@ -517,19 +522,6 @@ sub delete_menu_item
   }
 }
 
-sub configure {
-
-    # Process ColorEditor configuration options now.
-
-    my($objref, @hook_list) = @_;
-
-    my($option, $value);
-    while (($option, $value) = splice(@hook_list, 0, 2)) {
-	$objref->SUPER::configure($option => $value);
-    } # whilend all options/values
-
-} # end configure
-
 sub delete_widgets {
 
     # Remove widgets from consideration by the color configurator.
