@@ -400,13 +400,11 @@ ForceList(Tcl_Interp *interp, Tcl_Obj *sv)
             {
              if (*s == '{')
               count++;
-             else if (*s == '}' && (count-- <= 0))
+             else if (*s == '}' && (--count <= 0))
               break;
              s++;
             }
-           if (*s == '}')
-            s++; 
-           else
+           if (*s != '}')
             {
              /* Found end of string before closing '}' 
                 TCL would set an error, we will just include the
@@ -426,6 +424,8 @@ ForceList(Tcl_Interp *interp, Tcl_Obj *sv)
             }
           }
          av_store(av,i++,newSVpvn(base,(s-base)));
+         if (*s == '}')
+          s++; 
         }
       }
      /* Now have an AV populated decide how to return */                     
