@@ -8,7 +8,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkGrid.c 1.39 97/10/10 10:12:03
+ * RCS: @(#) $Id: tkGrid.c,v 1.3 1999/01/06 21:10:46 stanton Exp $
  */
 
 #include "tkInt.h"
@@ -440,6 +440,9 @@ Tk_GridCmd(clientData, interp, argc, argv)
 		    slavePtr->padX = slavePtr->padY = 0;
 		    slavePtr->iPadX = slavePtr->iPadY = 0;
 		    slavePtr->doubleBw = 2*Tk_Changes(tkwin)->border_width;
+		    if (slavePtr->flags & REQUESTED_RELAYOUT) {
+			Tcl_CancelIdleCall(ArrangeGrid, (ClientData) slavePtr);
+		    }
 		    slavePtr->flags = 0;
 		    slavePtr->sticky = 0;
 	    	}
@@ -710,7 +713,6 @@ Tk_GridCmd(clientData, interp, argc, argv)
 	Arg *indexP;		/* String value of current index list item. */
 	int ok;			/* temporary TCL result code */
 	int i;
-        LangFreeProc *freeProc = NULL;
 
 	if (((argc%2 != 0) && (argc>6)) || (argc < 4)) {
 	    Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
@@ -724,7 +726,7 @@ Tk_GridCmd(clientData, interp, argc, argv)
 	    return TCL_ERROR;
 	}
 
-	if (Lang_SplitList(interp, args[3], &argcPtr, &argsPtr, &freeProc) != TCL_OK) {
+	if (Tcl_ListObjGetElements(interp, args[3], &argcPtr, &argsPtr) != TCL_OK) {
 	    return TCL_ERROR;
 	}
 
@@ -735,8 +737,6 @@ Tk_GridCmd(clientData, interp, argc, argv)
 	    Tcl_AppendResult(interp, argv[3],
 		    " must be a single element.", (char *) NULL);
         problem:
-            if (freeProc)
-               (*freeProc)(argcPtr, argsPtr);
 	    return TCL_ERROR;
 	}
 	for (indexP=argsPtr; indexP < (argsPtr + argcPtr); indexP++) {
@@ -761,8 +761,6 @@ Tk_GridCmd(clientData, interp, argc, argv)
 	     */
 
 	    if (argc == 4) {
-		if (freeProc)
-		    (*freeProc)(argcPtr, argsPtr);
 		Tcl_ResetResult(interp);
 		Tcl_AppendElement(interp,"-minsize");
 		Tcl_IntResults(interp, 1, 1, (ok == TCL_OK) ? slotPtr[slot].minSize : 0);
@@ -835,8 +833,6 @@ Tk_GridCmd(clientData, interp, argc, argv)
 		}
 	    }
 	}
-	if (freeProc)
-	    (*freeProc)(argcPtr, argsPtr);
 
 	/*
 	 * If we changed a property, re-arrange the table,
