@@ -10,10 +10,6 @@
 #include "pTk/tkOption_f.h"
 #endif
 
-#ifndef dTHR
-#define dTHR int maybeTHR
-#endif
-
 typedef struct EventAndKeySym
  {XEvent event;
   KeySym keySym;
